// TimerWindow.cpp : implementation file
//

#include "stdafx.h"
#include "AutoSave.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static char *KeyAddIns   = "Software\\Microsoft\\DevStudio\\6.0\\AddIns";
static char *KeyAutoSave = "AutoSave.DSAddIn.1";
static char *KeyFull     = "Software\\Microsoft\\DevStudio\\6.0\\AddIns\\AutoSave.DSAddIn.1";

/////////////////////////////////////////////////////////////////////////////
// TimerWindow

TimerWindow::TimerWindow()
{
	m_saveFiles = true;
	m_saveWorkspace = true;
	m_protectIntellisense = true;
	m_period = 10;
	m_timer = 0;

	// Read the initial values from the registry
	CRegKey reg;
	// If it's the first time, an AddIns subkey may not exist, so we need to create it
	if (reg.Create(HKEY_CURRENT_USER, KeyAddIns) == ERROR_SUCCESS)
	{
		HKEY key = reg.Detach();
		if (reg.Create(key, KeyAutoSave) == ERROR_SUCCESS)
		{
			DWORD val;
			if (reg.QueryValue(val, "Enable") == ERROR_SUCCESS)
				m_saveFiles = val != 0;
			if (reg.QueryValue(val, "EnableWorkspace") == ERROR_SUCCESS)
				m_saveWorkspace = val != 0;
			if (reg.QueryValue(val, "ProtectIntellisense") == ERROR_SUCCESS)
				m_protectIntellisense = val != 0;
			if (reg.QueryValue(val, "Period") == ERROR_SUCCESS)
				m_period = val;
		}
		reg.Close();
		reg.Attach(key);
	}
}

TimerWindow::~TimerWindow()
{
	// Save the final parameters in the registry
	SaveParameters();
}

void TimerWindow::SaveParameters()
{
	// Save the final parameters in the registry
	CRegKey reg;
	if (reg.Create(HKEY_CURRENT_USER, KeyFull) == ERROR_SUCCESS)
	{
		reg.SetValue(m_saveFiles ? 1 : 0, "Enable");
		reg.SetValue(m_saveWorkspace ? 1 : 0, "EnableWorkspace");
		reg.SetValue(m_protectIntellisense ? 1 : 0, "ProtectIntellisense");
		reg.SetValue(m_period, "Period");
	}
}

BEGIN_MESSAGE_MAP(TimerWindow, CFrameWnd)
	//{{AFX_MSG_MAP(TimerWindow)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// TimerWindow message handlers

int TimerWindow::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// Start the timer (if enabled)
	ParameterChanged();
	
	return 0;
}

void TimerWindow::OnDestroy() 
{
	CFrameWnd::OnDestroy();
	
	if (m_timer)
	{
		KillTimer(m_timer);
		m_timer = 0;
	}
}

void TimerWindow::setSaveFiles(bool enable)
{
	if (m_saveFiles == enable)
		return;

	m_saveFiles = enable;
	ParameterChanged();
}

void TimerWindow::setSaveWorkspace(bool enable)
{
	if (m_saveWorkspace == enable)
		return;

	m_saveWorkspace = enable;
	ParameterChanged();
}

void TimerWindow::setProtectIntellisense(bool enable)
{
	m_protectIntellisense = enable;
}

void TimerWindow::setPeriod(DWORD period)
{
	if (m_period == period)
		return;

	m_period = period;
	ParameterChanged();
}

void TimerWindow::OnTimer(UINT nIDEvent) 
{
	CFrameWnd::OnTimer(nIDEvent);

	// Tip by: Tadeusz Dracz
	// Before saving the documents or the workspace it is necessary to get rid of the
	// IntelliSense Complete Word popup. It can be achieved by simply setting a focus
	// to a different window or make an other window active for a moment. Because the
	// popup is very sensitive to these actions it will dissappear.
	// It isn't however the perfect solution because the main Dev Studio window
	// flickers before saving but it works at least.
	if (m_protectIntellisense && (m_saveFiles || m_saveWorkspace))
	{
		HWND hForegroundWindow = ::GetForegroundWindow();
		::SetForegroundWindow(::GetDesktopWindow());
		::SetForegroundWindow(hForegroundWindow);
	}

	// Save the files (if enabled)
	if (m_saveFiles)
	{
		IDocuments *pDocuments;
		m_pApplication->get_Documents((IDispatch **)&pDocuments);
		if (pDocuments)
		{
			DsSaveStatus saved;
			pDocuments->SaveAll(_variant_t(false), &saved);
		}
		pDocuments->Release();
	}

	// Save the workspace (if enabled)
	if (m_saveWorkspace)
	{
		IGenericProject *pProject = NULL;
		m_pApplication->get_ActiveProject((IDispatch **)&pProject);
		if (pProject)
		{
			m_pApplication->ExecuteCommand(L"WorkspaceSave");
			pProject->Release();
		}
	}
}

void TimerWindow::ParameterChanged()
{
	if (m_timer && ((!m_saveFiles && !m_saveWorkspace) || !m_period))
	{
		// Stop saving
		KillTimer(m_timer);
		m_timer = 0;
	}
	else if ((m_saveFiles || m_saveWorkspace) && m_period)
	{
		// Kill the previous timer
		if (m_timer)
		{
			KillTimer(m_timer);
			m_timer = 0;
		}

		// Start generating save events (time is calculated in minutes)
		m_timer = SetTimer(1, m_period * 60000, NULL);
	}

	// Store the changed parameters in the registry
	SaveParameters();
}
